`timescale 1ns / 1ps
`include "defines.vh"
module unitTest(
        input wire clk,
        input wire reset,
		  output reg result
`ifdef DEBUG
		  ,
			wire [`TEST_I_ADDR_WIDTH-1:0] debug_program_counter,
			wire [`STAGE_COUNT-1:0]  debug_pipeline_stage
`endif
    );
	 
`ifdef DEBUG
	wire [`TEST_DATA_WIDTH-1:0] debug_alu_rr;
	wire [`TEST_DATA_WIDTH-1:0] debug_alu_rd;
	wire [`TEST_DATA_WIDTH-1:0] debug_alu_out;
	
   wire [`TEST_R_ADDR_WIDTH-1:0] debug_rr_addr;
   wire [`TEST_R_ADDR_WIDTH-1:0] debug_rd_addr;
	wire [`FLAG_COUNT-1:0] debug_flags_out;
	wire [`TEST_D_ADDR_WIDTH-1:0] debug_bus_address;
	wire [`TEST_D_ADDR_WIDTH-1:0] debug_register_Y;
	wire [`TEST_DATA_WIDTH-1:0] debug_bus_data;
	wire [11:0] debug_opcode_imd;
	
   wire [`OPCODE_COUNT-1:0] debug_opcode_type;
   wire [`GROUP_COUNT-1:0] debug_opcode_group;
	wire [`TEST_DATA_WIDTH-1:0] debug_writeback_value;
	wire [`SIGNAL_COUNT-1:0] debug_signals;
	
   reg [`TEST_I_ADDR_WIDTH-1:0] last_pc;
`endif


	// Instantiate the Unit Under Test (UUT)
	cpu #(
		.INSTR_WIDTH     (`TEST_INSTR_WIDTH),
      .DATA_WIDTH      (`TEST_DATA_WIDTH),
      .I_ADDR_WIDTH    (`TEST_I_ADDR_WIDTH),
      .RST_ACTIVE_LEVEL(1)
		) uut (
		.clk(clk), 
		.reset(reset)
`ifdef DEBUG
		,
		.debug_pipeline_stage(debug_pipeline_stage),
		.debug_program_counter(debug_program_counter),
		.debug_alu_rr(debug_alu_rr),
		.debug_alu_rd(debug_alu_rd),
		.debug_alu_out(debug_alu_out),
		.debug_flags_out(debug_flags_out),
		.debug_bus_address(debug_bus_address),
		.debug_register_Y(debug_register_Y),
		.debug_bus_data(debug_bus_data),
		.debug_opcode_type(debug_opcode_type),
		.debug_opcode_group(debug_opcode_group),
		.debug_opcode_imd(debug_opcode_imd),
		.debug_writeback_value(debug_writeback_value),
		.debug_signals(debug_signals),
		.debug_rd_addr(debug_rd_addr),
		.debug_rr_addr(debug_rr_addr)
`endif
	);
	
`include "unit_tests.vh"
	integer a1;
	integer a2;
	
	initial begin
		a1 = 0;
		a2 = 100;
	end
	
	always @(*) begin

		case(debug_program_counter)

			0: /*	 ldi 	r16, 0 		*/
				result = TEST_LDI(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										32'dX, 32'dX, 16, 0);
			1: /*	 ldi 	r17, 1 		*/
				result = TEST_LDI(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										32'dX, 32'dX, 17, 1);
			2: /*	 ldi 	r31, 100 		*/
				result = TEST_LDI(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										32'dX, 32'dX, 31, 100);
			3: /*	 ldi 	r30, 10 		*/
				result = TEST_LDI(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										32'dX, 32'dX, 30, 10);
			4: /*	 sub 	r31, r30 		*/
				begin 
				if(debug_program_counter != last_pc) begin
					last_pc = debug_program_counter;
					a2 = a2 -10;
				end
				result = TEST_SUB(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										32'dX, 30, 31, a2);
				end
			5: /*	 brmi 	enddiv 		*/
				result = TEST_BRBS(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										32'dX, 32'dX, 32'dX, 32'dX);
			6: /*	 breq 	enddiv 		*/
				result = TEST_BRBS(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										32'dX, 32'dX, 32'dX, 32'dX);
			7: /*	 add 	r16, r17 		*/
			begin
				if(debug_program_counter != last_pc) begin
					last_pc = debug_program_counter;
					a1 = a1 +1;
				end
				result = TEST_ADD(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										32'dX, 17, 16, a1);
			end
			8: /*	 rjmp 	begindiv 		*/
				result = TEST_RJMP(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										32'dX, 32'dX, 32'dX, 32'dX);
			9: /*	 sts 	r16, 0 		*/
				result = TEST_STS(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										0, 16, 32'dX, 10);
			10: /*	 lds 	r20, 0 		*/
				result = TEST_LDS(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										0, 32'dX, 20, 10);
			11: /*	 push 	r20 		*/
				result = TEST_PUSH(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										32'dX, 32'dX, 32'dX, 32'dX);
			12: /*	 pop 	r21 		*/
				result = TEST_POP(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										32'dX, 32'dX, 32'dX, 32'dX);
			13: /*	 mov 	r22, r21 		*/
				result = TEST_MOV(debug_pipeline_stage, debug_opcode_group, debug_opcode_type, debug_opcode_imd, debug_writeback_value, 
										debug_signals, debug_rr_addr, debug_rd_addr, debug_alu_rr, debug_alu_rd, debug_alu_out, debug_bus_address,
										32'dX, 21, 22, 10);
				
			default:
				result <= 1'bz;
		endcase
		last_pc = debug_program_counter;
	end
endmodule
